package org.eclipse.jdt.internal.compiler.ast;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.Assignment;
import org.eclipse.jdt.internal.compiler.ast.CompoundAssignment;
import org.eclipse.jdt.internal.compiler.ast.Expression;
import org.eclipse.jdt.internal.compiler.ast.IntLiteral;
import org.eclipse.jdt.internal.compiler.ast.NameReference;
import org.eclipse.jdt.internal.compiler.ast.Reference;
import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
import org.eclipse.jdt.internal.compiler.flow.FlowContext;
import org.eclipse.jdt.internal.compiler.flow.FlowInfo;
import org.eclipse.jdt.internal.compiler.flow.UnconditionalFlowInfo;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.BaseTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.ClassScope;
import org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
import org.eclipse.jdt.internal.compiler.lookup.InvocationSite;
import org.eclipse.jdt.internal.compiler.lookup.LocalVariableBinding;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.MethodScope;
import org.eclipse.jdt.internal.compiler.lookup.ProblemFieldBinding;
import org.eclipse.jdt.internal.compiler.lookup.ProblemReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.SourceTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.SyntheticMethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.VariableBinding;

public class QualifiedNameReference extends NameReference {
  public char[][] tokens;
  
  public long[] sourcePositions;
  
  public FieldBinding[] otherBindings;
  
  int[] otherDepths;
  
  public int indexOfFirstFieldBinding;
  
  public SyntheticMethodBinding syntheticWriteAccessor;
  
  public SyntheticMethodBinding[] syntheticReadAccessors;
  
  public TypeBinding genericCast;
  
  public TypeBinding[] otherGenericCasts;
  
  public QualifiedNameReference(char[][] tokens, long[] positions, int sourceStart, int sourceEnd) {
    this.tokens = tokens;
    this.sourcePositions = positions;
    this.sourceStart = sourceStart;
    this.sourceEnd = sourceEnd;
  }
  
  public FlowInfo analyseAssignment(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo, Assignment assignment, boolean isCompound) {
    UnconditionalFlowInfo unconditionalFlowInfo;
    LocalVariableBinding localBinding;
    int otherBindingsCount = (this.otherBindings == null) ? 0 : this.otherBindings.length;
    boolean needValue = !(otherBindingsCount != 0 && this.otherBindings[0].isStatic());
    boolean complyTo14 = ((currentScope.compilerOptions()).complianceLevel >= 3145728L);
    FieldBinding lastFieldBinding = null;
    switch (this.bits & 0x7) {
      case 1:
        lastFieldBinding = (FieldBinding)this.binding;
        if (needValue || complyTo14)
          manageSyntheticAccessIfNecessary(currentScope, lastFieldBinding, 0, flowInfo); 
        if (lastFieldBinding.isBlankFinal() && 
          this.otherBindings != null && 
          currentScope.needBlankFinalFieldInitializationCheck(lastFieldBinding)) {
          FlowInfo fieldInits = flowContext.getInitsForFinalBlankInitializationCheck(lastFieldBinding.declaringClass.original(), flowInfo);
          if (!fieldInits.isDefinitelyAssigned(lastFieldBinding))
            currentScope.problemReporter().uninitializedBlankFinalField(lastFieldBinding, (ASTNode)this); 
        } 
        break;
      case 2:
        if (!flowInfo.isDefinitelyAssigned(localBinding = (LocalVariableBinding)this.binding))
          currentScope.problemReporter().uninitializedLocalVariable(localBinding, (ASTNode)this); 
        if ((flowInfo.tagBits & 0x3) == 0) {
          localBinding.useFlag = 1;
        } else if (localBinding.useFlag == 0) {
          localBinding.useFlag = 2;
        } 
        if (needValue)
          checkInternalNPE(currentScope, flowContext, flowInfo, true); 
        break;
    } 
    if (needValue)
      manageEnclosingInstanceAccessIfNecessary(currentScope, flowInfo); 
    if (this.otherBindings != null) {
      for (int i = 0; i < otherBindingsCount - 1; i++) {
        lastFieldBinding = this.otherBindings[i];
        needValue = !this.otherBindings[i + 1].isStatic();
        if (needValue || complyTo14)
          manageSyntheticAccessIfNecessary(currentScope, lastFieldBinding, i + 1, flowInfo); 
      } 
      lastFieldBinding = this.otherBindings[otherBindingsCount - 1];
    } 
    if (isCompound) {
      if (otherBindingsCount == 0 && 
        lastFieldBinding.isBlankFinal() && 
        currentScope.needBlankFinalFieldInitializationCheck(lastFieldBinding)) {
        FlowInfo fieldInits = flowContext.getInitsForFinalBlankInitializationCheck((TypeBinding)lastFieldBinding.declaringClass, flowInfo);
        if (!fieldInits.isDefinitelyAssigned(lastFieldBinding))
          currentScope.problemReporter().uninitializedBlankFinalField(lastFieldBinding, (ASTNode)this); 
      } 
      manageSyntheticAccessIfNecessary(currentScope, lastFieldBinding, otherBindingsCount, flowInfo);
    } 
    if (assignment.expression != null)
      unconditionalFlowInfo = 
        assignment
        .expression
        .analyseCode(currentScope, flowContext, flowInfo)
        .unconditionalInits(); 
    if (lastFieldBinding.isFinal())
      if (otherBindingsCount == 0 && 
        this.indexOfFirstFieldBinding == 1 && 
        lastFieldBinding.isBlankFinal() && 
        !isCompound && 
        currentScope.allowBlankFinalFieldAssignment(lastFieldBinding)) {
        if (unconditionalFlowInfo.isPotentiallyAssigned(lastFieldBinding)) {
          currentScope.problemReporter().duplicateInitializationOfBlankFinalField(lastFieldBinding, (Reference)this);
        } else {
          flowContext.recordSettingFinal((VariableBinding)lastFieldBinding, (Reference)this, (FlowInfo)unconditionalFlowInfo);
        } 
        unconditionalFlowInfo.markAsDefinitelyAssigned(lastFieldBinding);
      } else {
        currentScope.problemReporter().cannotAssignToFinalField(lastFieldBinding, (ASTNode)this);
        if (otherBindingsCount == 0 && currentScope.allowBlankFinalFieldAssignment(lastFieldBinding))
          unconditionalFlowInfo.markAsDefinitelyAssigned(lastFieldBinding); 
      }  
    manageSyntheticAccessIfNecessary(currentScope, lastFieldBinding, -1, (FlowInfo)unconditionalFlowInfo);
    return (FlowInfo)unconditionalFlowInfo;
  }
  
  public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo) {
    return analyseCode(currentScope, flowContext, flowInfo, true);
  }
  
  public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo, boolean valueRequired) {
    FieldBinding fieldBinding;
    LocalVariableBinding localBinding;
    int otherBindingsCount = (this.otherBindings == null) ? 0 : this.otherBindings.length;
    boolean needValue = (otherBindingsCount == 0) ? valueRequired : (!this.otherBindings[0].isStatic());
    boolean complyTo14 = ((currentScope.compilerOptions()).complianceLevel >= 3145728L);
    switch (this.bits & 0x7) {
      case 1:
        if (needValue || complyTo14)
          manageSyntheticAccessIfNecessary(currentScope, (FieldBinding)this.binding, 0, flowInfo); 
        fieldBinding = (FieldBinding)this.binding;
        if (this.indexOfFirstFieldBinding == 1)
          if (fieldBinding.isBlankFinal() && 
            currentScope.needBlankFinalFieldInitializationCheck(fieldBinding)) {
            FlowInfo fieldInits = flowContext.getInitsForFinalBlankInitializationCheck(fieldBinding.declaringClass.original(), flowInfo);
            if (!fieldInits.isDefinitelyAssigned(fieldBinding))
              currentScope.problemReporter().uninitializedBlankFinalField(fieldBinding, (ASTNode)this); 
          }  
        break;
      case 2:
        if (!flowInfo.isDefinitelyAssigned(localBinding = (LocalVariableBinding)this.binding))
          currentScope.problemReporter().uninitializedLocalVariable(localBinding, (ASTNode)this); 
        if ((flowInfo.tagBits & 0x3) == 0) {
          localBinding.useFlag = 1;
          break;
        } 
        if (localBinding.useFlag == 0)
          localBinding.useFlag = 2; 
        break;
    } 
    if (needValue)
      checkInternalNPE(currentScope, flowContext, flowInfo, true); 
    if (needValue)
      manageEnclosingInstanceAccessIfNecessary(currentScope, flowInfo); 
    if (this.otherBindings != null)
      for (int i = 0; i < otherBindingsCount; i++) {
        needValue = (i < otherBindingsCount - 1) ? (!this.otherBindings[i + 1].isStatic()) : valueRequired;
        if (needValue || complyTo14)
          manageSyntheticAccessIfNecessary(currentScope, this.otherBindings[i], i + 1, flowInfo); 
      }  
    return flowInfo;
  }
  
  private void checkInternalNPE(BlockScope scope, FlowContext flowContext, FlowInfo flowInfo, boolean checkString) {
    if ((this.bits & 0x7) == 2) {
      LocalVariableBinding local = (LocalVariableBinding)this.binding;
      if (local != null && (
        local.type.tagBits & 0x2L) == 0L && (
        checkString || local.type.id != 11)) {
        if ((this.bits & 0x20000) == 0)
          flowContext.recordUsingNullReference((Scope)scope, local, (ASTNode)this, 
              3, flowInfo); 
        flowInfo.markAsComparedEqualToNonNull(local);
        flowContext.markFinallyNullStatus(local, 4);
      } 
    } 
    if (this.otherBindings != null) {
      if ((this.bits & 0x7) == 1)
        checkNullableFieldDereference((Scope)scope, (FieldBinding)this.binding, this.sourcePositions[this.indexOfFirstFieldBinding - 1], flowContext, 0); 
      int length = this.otherBindings.length - 1;
      for (int i = 0; i < length; i++)
        checkNullableFieldDereference((Scope)scope, this.otherBindings[i], this.sourcePositions[this.indexOfFirstFieldBinding + i], flowContext, 0); 
    } 
  }
  
  public boolean checkNPE(BlockScope scope, FlowContext flowContext, FlowInfo flowInfo, int ttlForFieldCheck) {
    if (super.checkNPE(scope, flowContext, flowInfo, ttlForFieldCheck))
      return true; 
    FieldBinding fieldBinding = null;
    long position = 0L;
    if (this.otherBindings == null) {
      if ((this.bits & 0x7) == 1) {
        fieldBinding = (FieldBinding)this.binding;
        position = this.sourcePositions[0];
      } 
    } else {
      fieldBinding = this.otherBindings[this.otherBindings.length - 1];
      position = this.sourcePositions[this.sourcePositions.length - 1];
    } 
    if (fieldBinding != null)
      return checkNullableFieldDereference((Scope)scope, fieldBinding, position, flowContext, ttlForFieldCheck); 
    return false;
  }
  
  public void computeConversion(Scope scope, TypeBinding runtimeTimeType, TypeBinding compileTimeType) {
    if (runtimeTimeType == null || compileTimeType == null)
      return; 
    FieldBinding field = null;
    int length = (this.otherBindings == null) ? 0 : this.otherBindings.length;
    if (length == 0) {
      if ((this.bits & 0x1) != 0 && this.binding != null && this.binding.isValidBinding())
        field = (FieldBinding)this.binding; 
    } else {
      field = this.otherBindings[length - 1];
    } 
    if (field != null) {
      FieldBinding originalBinding = field.original();
      TypeBinding originalType = originalBinding.type;
      if (originalType.leafComponentType().isTypeVariable()) {
        TypeBinding targetType = (!compileTimeType.isBaseType() && runtimeTimeType.isBaseType()) ? 
          compileTimeType : 
          runtimeTimeType;
        TypeBinding typeCast = originalType.genericCast(targetType);
        setGenericCast(length, typeCast);
        if (typeCast instanceof ReferenceBinding) {
          ReferenceBinding referenceCast = (ReferenceBinding)typeCast;
          if (!referenceCast.canBeSeenBy(scope))
            scope.problemReporter().invalidType((ASTNode)this, 
                (TypeBinding)new ProblemReferenceBinding(
                  CharOperation.splitOn('.', referenceCast.shortReadableName()), 
                  referenceCast, 
                  2)); 
        } 
      } 
    } 
    super.computeConversion(scope, runtimeTimeType, compileTimeType);
  }
  
  public void generateAssignment(BlockScope currentScope, CodeStream codeStream, Assignment assignment, boolean valueRequired) {
    int pc = codeStream.position;
    FieldBinding lastFieldBinding = generateReadSequence(currentScope, codeStream);
    codeStream.recordPositionsFrom(pc, this.sourceStart);
    assignment.expression.generateCode(currentScope, codeStream, true);
    fieldStore((Scope)currentScope, codeStream, lastFieldBinding, (MethodBinding)this.syntheticWriteAccessor, getFinalReceiverType(), false, valueRequired);
    if (valueRequired)
      codeStream.generateImplicitConversion(assignment.implicitConversion); 
  }
  
  public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean valueRequired) {
    int pc = codeStream.position;
    if (this.constant != Constant.NotAConstant) {
      if (valueRequired)
        codeStream.generateConstant(this.constant, this.implicitConversion); 
    } else {
      FieldBinding lastFieldBinding = generateReadSequence(currentScope, codeStream);
      if (lastFieldBinding != null) {
        boolean isStatic = lastFieldBinding.isStatic();
        Constant fieldConstant = lastFieldBinding.constant();
        if (fieldConstant != Constant.NotAConstant) {
          if (!isStatic) {
            codeStream.invokeObjectGetClass();
            codeStream.pop();
          } 
          if (valueRequired)
            codeStream.generateConstant(fieldConstant, this.implicitConversion); 
        } else {
          boolean isFirst = (lastFieldBinding == this.binding && (
            this.indexOfFirstFieldBinding == 1 || TypeBinding.equalsEquals((TypeBinding)lastFieldBinding.declaringClass, (TypeBinding)currentScope.enclosingReceiverType())) && 
            this.otherBindings == null);
          TypeBinding requiredGenericCast = getGenericCast((this.otherBindings == null) ? 0 : this.otherBindings.length);
          if (valueRequired || (
            !isFirst && (currentScope.compilerOptions()).complianceLevel >= 3145728L) || (
            this.implicitConversion & 0x400) != 0 || 
            requiredGenericCast != null) {
            int lastFieldPc = codeStream.position;
            if (lastFieldBinding.declaringClass == null) {
              codeStream.arraylength();
              if (valueRequired) {
                codeStream.generateImplicitConversion(this.implicitConversion);
              } else {
                codeStream.pop();
              } 
            } else {
              SyntheticMethodBinding accessor = (this.syntheticReadAccessors == null) ? null : this.syntheticReadAccessors[this.syntheticReadAccessors.length - 1];
              if (accessor == null) {
                TypeBinding constantPoolDeclaringClass = CodeStream.getConstantPoolDeclaringClass((Scope)currentScope, lastFieldBinding, getFinalReceiverType(), isFirst);
                if (isStatic) {
                  codeStream.fieldAccess((byte)-78, lastFieldBinding, constantPoolDeclaringClass);
                } else {
                  codeStream.fieldAccess((byte)-76, lastFieldBinding, constantPoolDeclaringClass);
                } 
              } else {
                codeStream.invoke((byte)-72, (MethodBinding)accessor, null);
              } 
              if (requiredGenericCast != null)
                codeStream.checkcast(requiredGenericCast); 
              if (valueRequired) {
                codeStream.generateImplicitConversion(this.implicitConversion);
              } else {
                boolean isUnboxing = ((this.implicitConversion & 0x400) != 0);
                if (isUnboxing)
                  codeStream.generateImplicitConversion(this.implicitConversion); 
                switch (isUnboxing ? (postConversionType((Scope)currentScope)).id : lastFieldBinding.type.id) {
                  case 7:
                  case 8:
                    codeStream.pop2();
                    break;
                  default:
                    codeStream.pop();
                    break;
                } 
              } 
            } 
            int fieldPosition = (int)(this.sourcePositions[this.sourcePositions.length - 1] >>> 32L);
            codeStream.recordPositionsFrom(lastFieldPc, fieldPosition);
          } else if (!isStatic) {
            codeStream.invokeObjectGetClass();
            codeStream.pop();
          } 
        } 
      } 
    } 
    codeStream.recordPositionsFrom(pc, this.sourceStart);
  }
  
  public void generateCompoundAssignment(BlockScope currentScope, CodeStream codeStream, Expression expression, int operator, int assignmentImplicitConversion, boolean valueRequired) {
    TypeBinding requiredGenericCast;
    FieldBinding lastFieldBinding = generateReadSequence(currentScope, codeStream);
    reportOnlyUselesslyReadPrivateField(currentScope, lastFieldBinding, valueRequired);
    boolean isFirst = (lastFieldBinding == this.binding && (
      this.indexOfFirstFieldBinding == 1 || TypeBinding.equalsEquals((TypeBinding)lastFieldBinding.declaringClass, (TypeBinding)currentScope.enclosingReceiverType())) && 
      this.otherBindings == null);
    TypeBinding constantPoolDeclaringClass = CodeStream.getConstantPoolDeclaringClass((Scope)currentScope, lastFieldBinding, getFinalReceiverType(), isFirst);
    SyntheticMethodBinding accessor = (this.syntheticReadAccessors == null) ? null : this.syntheticReadAccessors[this.syntheticReadAccessors.length - 1];
    if (lastFieldBinding.isStatic()) {
      if (accessor == null) {
        codeStream.fieldAccess((byte)-78, lastFieldBinding, constantPoolDeclaringClass);
      } else {
        codeStream.invoke((byte)-72, (MethodBinding)accessor, null);
      } 
    } else {
      codeStream.dup();
      if (accessor == null) {
        codeStream.fieldAccess((byte)-76, lastFieldBinding, constantPoolDeclaringClass);
      } else {
        codeStream.invoke((byte)-72, (MethodBinding)accessor, null);
      } 
    } 
    int operationTypeID;
    switch (operationTypeID = (this.implicitConversion & 0xFF) >> 4) {
      case 0:
      case 1:
      case 11:
        codeStream.generateStringConcatenationAppend(currentScope, null, expression);
        break;
      default:
        requiredGenericCast = getGenericCast((this.otherBindings == null) ? 0 : this.otherBindings.length);
        if (requiredGenericCast != null)
          codeStream.checkcast(requiredGenericCast); 
        codeStream.generateImplicitConversion(this.implicitConversion);
        if (expression == IntLiteral.One) {
          codeStream.generateConstant(expression.constant, this.implicitConversion);
        } else {
          expression.generateCode(currentScope, codeStream, true);
        } 
        codeStream.sendOperator(operator, operationTypeID);
        codeStream.generateImplicitConversion(assignmentImplicitConversion);
        break;
    } 
    fieldStore((Scope)currentScope, codeStream, lastFieldBinding, (MethodBinding)this.syntheticWriteAccessor, getFinalReceiverType(), false, valueRequired);
  }
  
  public void generatePostIncrement(BlockScope currentScope, CodeStream codeStream, CompoundAssignment postIncrement, boolean valueRequired) {
    TypeBinding operandType;
    FieldBinding lastFieldBinding = generateReadSequence(currentScope, codeStream);
    reportOnlyUselesslyReadPrivateField(currentScope, lastFieldBinding, valueRequired);
    boolean isFirst = (lastFieldBinding == this.binding && (
      this.indexOfFirstFieldBinding == 1 || TypeBinding.equalsEquals((TypeBinding)lastFieldBinding.declaringClass, (TypeBinding)currentScope.enclosingReceiverType())) && 
      this.otherBindings == null);
    TypeBinding constantPoolDeclaringClass = CodeStream.getConstantPoolDeclaringClass((Scope)currentScope, lastFieldBinding, getFinalReceiverType(), isFirst);
    SyntheticMethodBinding accessor = (this.syntheticReadAccessors == null) ? 
      null : 
      this.syntheticReadAccessors[this.syntheticReadAccessors.length - 1];
    if (lastFieldBinding.isStatic()) {
      if (accessor == null) {
        codeStream.fieldAccess((byte)-78, lastFieldBinding, constantPoolDeclaringClass);
      } else {
        codeStream.invoke((byte)-72, (MethodBinding)accessor, constantPoolDeclaringClass);
      } 
    } else {
      codeStream.dup();
      if (accessor == null) {
        codeStream.fieldAccess((byte)-76, lastFieldBinding, null);
      } else {
        codeStream.invoke((byte)-72, (MethodBinding)accessor, null);
      } 
    } 
    TypeBinding requiredGenericCast = getGenericCast((this.otherBindings == null) ? 0 : this.otherBindings.length);
    if (requiredGenericCast != null) {
      codeStream.checkcast(requiredGenericCast);
      operandType = requiredGenericCast;
    } else {
      operandType = lastFieldBinding.type;
    } 
    if (valueRequired)
      if (lastFieldBinding.isStatic()) {
        switch (operandType.id) {
          case 7:
          case 8:
            codeStream.dup2();
            break;
          default:
            codeStream.dup();
            break;
        } 
      } else {
        switch (operandType.id) {
          case 7:
          case 8:
            codeStream.dup2_x1();
            break;
          default:
            codeStream.dup_x1();
            break;
        } 
      }  
    codeStream.generateImplicitConversion(this.implicitConversion);
    codeStream.generateConstant(
        postIncrement.expression.constant, 
        this.implicitConversion);
    codeStream.sendOperator(postIncrement.operator, this.implicitConversion & 0xF);
    codeStream.generateImplicitConversion(
        postIncrement.preAssignImplicitConversion);
    fieldStore((Scope)currentScope, codeStream, lastFieldBinding, (MethodBinding)this.syntheticWriteAccessor, getFinalReceiverType(), false, false);
  }
  
  public FieldBinding generateReadSequence(BlockScope currentScope, CodeStream codeStream) {
    FieldBinding lastFieldBinding;
    TypeBinding lastGenericCast, lastReceiverType;
    LocalVariableBinding localBinding;
    Constant localConstant;
    int otherBindingsCount = (this.otherBindings == null) ? 0 : this.otherBindings.length;
    boolean needValue = !(otherBindingsCount != 0 && this.otherBindings[0].isStatic());
    boolean complyTo14 = ((currentScope.compilerOptions()).complianceLevel >= 3145728L);
    switch (this.bits & 0x7) {
      case 1:
        lastFieldBinding = ((FieldBinding)this.binding).original();
        lastGenericCast = this.genericCast;
        lastReceiverType = this.actualReceiverType;
        if (lastFieldBinding.constant() != Constant.NotAConstant)
          break; 
        if ((needValue && !lastFieldBinding.isStatic()) || lastGenericCast != null) {
          int pc = codeStream.position;
          if ((this.bits & 0x1FE0) != 0) {
            ReferenceBinding targetType = currentScope.enclosingSourceType().enclosingTypeAt((this.bits & 0x1FE0) >> 5);
            Object[] emulationPath = currentScope.getEmulationPath(targetType, true, false);
            codeStream.generateOuterAccess(emulationPath, (ASTNode)this, (Binding)targetType, (Scope)currentScope);
          } else {
            generateReceiver(codeStream);
          } 
          codeStream.recordPositionsFrom(pc, this.sourceStart);
        } 
        break;
      case 2:
        lastFieldBinding = null;
        lastGenericCast = null;
        localBinding = (LocalVariableBinding)this.binding;
        lastReceiverType = localBinding.type;
        if (!needValue)
          break; 
        localConstant = localBinding.constant();
        if (localConstant != Constant.NotAConstant) {
          codeStream.generateConstant(localConstant, 0);
          break;
        } 
        if ((this.bits & 0x80000) != 0) {
          checkEffectiveFinality(localBinding, (Scope)currentScope);
          VariableBinding[] path = currentScope.getEmulationPath(localBinding);
          codeStream.generateOuterAccess((Object[])path, (ASTNode)this, (Binding)localBinding, (Scope)currentScope);
          break;
        } 
        codeStream.load(localBinding);
        break;
      default:
        return null;
    } 
    int positionsLength = this.sourcePositions.length;
    FieldBinding initialFieldBinding = lastFieldBinding;
    if (this.otherBindings != null)
      for (int i = 0; i < otherBindingsCount; i++) {
        int pc = codeStream.position;
        FieldBinding nextField = this.otherBindings[i].original();
        TypeBinding nextGenericCast = (this.otherGenericCasts == null) ? null : this.otherGenericCasts[i];
        if (lastFieldBinding != null) {
          needValue = !nextField.isStatic();
          Constant fieldConstant = lastFieldBinding.constant();
          if (fieldConstant != Constant.NotAConstant) {
            if (i > 0 && !lastFieldBinding.isStatic()) {
              codeStream.invokeObjectGetClass();
              codeStream.pop();
            } 
            if (needValue)
              codeStream.generateConstant(fieldConstant, 0); 
          } else {
            if (needValue || (i > 0 && complyTo14) || lastGenericCast != null) {
              SyntheticMethodBinding syntheticMethodBinding = (this.syntheticReadAccessors == null) ? null : this.syntheticReadAccessors[i];
              if (syntheticMethodBinding == null) {
                TypeBinding constantPoolDeclaringClass = CodeStream.getConstantPoolDeclaringClass((Scope)currentScope, lastFieldBinding, lastReceiverType, (i == 0 && this.indexOfFirstFieldBinding == 1));
                if (lastFieldBinding.isStatic()) {
                  codeStream.fieldAccess((byte)-78, lastFieldBinding, constantPoolDeclaringClass);
                } else {
                  codeStream.fieldAccess((byte)-76, lastFieldBinding, constantPoolDeclaringClass);
                } 
              } else {
                codeStream.invoke((byte)-72, (MethodBinding)syntheticMethodBinding, null);
              } 
              if (lastGenericCast != null) {
                codeStream.checkcast(lastGenericCast);
                lastReceiverType = lastGenericCast;
              } else {
                lastReceiverType = lastFieldBinding.type;
              } 
              if (!needValue)
                codeStream.pop(); 
            } else {
              if (lastFieldBinding == initialFieldBinding) {
                if (lastFieldBinding.isStatic())
                  if (TypeBinding.notEquals((TypeBinding)initialFieldBinding.declaringClass, this.actualReceiverType.erasure())) {
                    SyntheticMethodBinding syntheticMethodBinding = (this.syntheticReadAccessors == null) ? null : this.syntheticReadAccessors[i];
                    if (syntheticMethodBinding == null) {
                      TypeBinding constantPoolDeclaringClass = CodeStream.getConstantPoolDeclaringClass((Scope)currentScope, lastFieldBinding, lastReceiverType, (i == 0 && this.indexOfFirstFieldBinding == 1));
                      codeStream.fieldAccess((byte)-78, lastFieldBinding, constantPoolDeclaringClass);
                    } else {
                      codeStream.invoke((byte)-72, (MethodBinding)syntheticMethodBinding, null);
                    } 
                    codeStream.pop();
                  }  
              } else if (!lastFieldBinding.isStatic()) {
                codeStream.invokeObjectGetClass();
                codeStream.pop();
              } 
              lastReceiverType = lastFieldBinding.type;
            } 
            if (positionsLength - otherBindingsCount + i - 1 >= 0) {
              int fieldPosition = (int)(this.sourcePositions[positionsLength - otherBindingsCount + i - 1] >>> 32L);
              codeStream.recordPositionsFrom(pc, fieldPosition);
            } 
          } 
        } 
        lastFieldBinding = nextField;
        lastGenericCast = nextGenericCast;
      }  
    return lastFieldBinding;
  }
  
  public void generateReceiver(CodeStream codeStream) {
    codeStream.aload_0();
  }
  
  public TypeBinding[] genericTypeArguments() {
    return null;
  }
  
  protected FieldBinding getCodegenBinding(int index) {
    if (index == 0)
      return ((FieldBinding)this.binding).original(); 
    return this.otherBindings[index - 1].original();
  }
  
  protected TypeBinding getFinalReceiverType() {
    int otherBindingsCount = (this.otherBindings == null) ? 0 : this.otherBindings.length;
    switch (otherBindingsCount) {
      case 0:
        return this.actualReceiverType;
      case 1:
        return (this.genericCast != null) ? this.genericCast : ((VariableBinding)this.binding).type;
    } 
    TypeBinding previousGenericCast = (this.otherGenericCasts == null) ? null : this.otherGenericCasts[otherBindingsCount - 2];
    return (previousGenericCast != null) ? previousGenericCast : (this.otherBindings[otherBindingsCount - 2]).type;
  }
  
  protected TypeBinding getGenericCast(int index) {
    if (index == 0)
      return this.genericCast; 
    if (this.otherGenericCasts == null)
      return null; 
    return this.otherGenericCasts[index - 1];
  }
  
  public TypeBinding getOtherFieldBindings(BlockScope scope) {
    int length = this.tokens.length;
    FieldBinding field = ((this.bits & 0x1) != 0) ? (FieldBinding)this.binding : null;
    TypeBinding type = ((VariableBinding)this.binding).type;
    int index = this.indexOfFirstFieldBinding;
    if (index == length) {
      this.constant = ((FieldBinding)this.binding).constant((Scope)scope);
      return (type != null && (this.bits & 0x2000) == 0) ? 
        type.capture((Scope)scope, this.sourceStart, this.sourceEnd) : 
        type;
    } 
    int otherBindingsLength = length - index;
    this.otherBindings = new FieldBinding[otherBindingsLength];
    this.otherDepths = new int[otherBindingsLength];
    this.constant = ((VariableBinding)this.binding).constant((Scope)scope);
    int firstDepth = (this.bits & 0x1FE0) >> 5;
    while (index < length) {
      char[] token = this.tokens[index];
      if (type == null)
        return null; 
      this.bits &= 0xFFFFE01F;
      FieldBinding previousField = field;
      field = scope.getField(type.capture((Scope)scope, (int)(this.sourcePositions[index] >>> 32L), (int)this.sourcePositions[index]), token, (InvocationSite)this);
      int place = index - this.indexOfFirstFieldBinding;
      this.otherBindings[place] = field;
      this.otherDepths[place] = (this.bits & 0x1FE0) >> 5;
      if (field.isValidBinding()) {
        if (previousField != null) {
          TypeBinding fieldReceiverType = type;
          TypeBinding oldReceiverType = fieldReceiverType;
          fieldReceiverType = fieldReceiverType.getErasureCompatibleType((TypeBinding)field.declaringClass);
          FieldBinding originalBinding = previousField.original();
          if (TypeBinding.notEquals(fieldReceiverType, oldReceiverType) || originalBinding.type.leafComponentType().isTypeVariable())
            setGenericCast(index - 1, originalBinding.type.genericCast(fieldReceiverType)); 
        } 
        if (isFieldUseDeprecated(field, (Scope)scope, (index + 1 == length) ? this.bits : 0))
          scope.problemReporter().deprecatedField(field, (ASTNode)this); 
        if (this.constant != Constant.NotAConstant)
          this.constant = field.constant((Scope)scope); 
        if (field.isStatic()) {
          if ((field.modifiers & 0x4000) != 0) {
            ReferenceBinding declaringClass = (field.original()).declaringClass;
            MethodScope methodScope = scope.methodScope();
            SourceTypeBinding sourceType = methodScope.enclosingSourceType();
            if ((this.bits & 0x2000) == 0 && 
              TypeBinding.equalsEquals((TypeBinding)sourceType, (TypeBinding)declaringClass) && 
              methodScope.lastVisibleFieldID >= 0 && 
              field.id >= methodScope.lastVisibleFieldID && (
              !field.isStatic() || methodScope.isStatic))
              scope.problemReporter().forwardReference((Reference)this, index, field); 
            if ((TypeBinding.equalsEquals((TypeBinding)sourceType, (TypeBinding)declaringClass) || TypeBinding.equalsEquals((TypeBinding)sourceType.superclass, (TypeBinding)declaringClass)) && 
              field.constant((Scope)scope) == Constant.NotAConstant && 
              !methodScope.isStatic && 
              methodScope.isInsideInitializerOrConstructor())
              scope.problemReporter().enumStaticFieldUsedDuringInitialization(field, (ASTNode)this); 
          } 
          scope.problemReporter().nonStaticAccessToStaticField((ASTNode)this, field, index);
          if (TypeBinding.notEquals((TypeBinding)field.declaringClass, type))
            scope.problemReporter().indirectAccessToStaticField((ASTNode)this, field); 
        } 
        type = field.type;
        index++;
        continue;
      } 
      this.constant = Constant.NotAConstant;
      scope.problemReporter().invalidField(this, field, index, type);
      setDepth(firstDepth);
      return null;
    } 
    setDepth(firstDepth);
    type = (this.otherBindings[otherBindingsLength - 1]).type;
    return (type != null && (this.bits & 0x2000) == 0) ? 
      type.capture((Scope)scope, this.sourceStart, this.sourceEnd) : 
      type;
  }
  
  public boolean isEquivalent(Reference reference) {
    if (reference instanceof org.eclipse.jdt.internal.compiler.ast.FieldReference)
      return reference.isEquivalent((Reference)this); 
    if (!(reference instanceof org.eclipse.jdt.internal.compiler.ast.QualifiedNameReference))
      return false; 
    org.eclipse.jdt.internal.compiler.ast.QualifiedNameReference qualifiedReference = (org.eclipse.jdt.internal.compiler.ast.QualifiedNameReference)reference;
    if (this.tokens.length != qualifiedReference.tokens.length)
      return false; 
    if (this.binding != qualifiedReference.binding)
      return false; 
    if (this.otherBindings != null) {
      if (qualifiedReference.otherBindings == null)
        return false; 
      int len = this.otherBindings.length;
      if (len != qualifiedReference.otherBindings.length)
        return false; 
      for (int i = 0; i < len; i++) {
        if (this.otherBindings[i] != qualifiedReference.otherBindings[i])
          return false; 
      } 
    } else if (qualifiedReference.otherBindings != null) {
      return false;
    } 
    return true;
  }
  
  public boolean isFieldAccess() {
    if (this.otherBindings != null)
      return true; 
    return ((this.bits & 0x7) == 1);
  }
  
  public FieldBinding lastFieldBinding() {
    if (this.otherBindings != null)
      return this.otherBindings[this.otherBindings.length - 1]; 
    if (this.binding != null && (this.bits & 0x7) == 1)
      return (FieldBinding)this.binding; 
    return null;
  }
  
  public void manageEnclosingInstanceAccessIfNecessary(BlockScope currentScope, FlowInfo flowInfo) {
    if (((this.bits & 0x1FE0) == 0 && (this.bits & 0x80000) == 0) || this.constant != Constant.NotAConstant)
      return; 
    if ((this.bits & 0x7) == 2) {
      LocalVariableBinding localVariableBinding = (LocalVariableBinding)this.binding;
      if (localVariableBinding != null) {
        if ((localVariableBinding.tagBits & 0x100L) != 0L)
          return; 
        switch (localVariableBinding.useFlag) {
          case 1:
          case 2:
            currentScope.emulateOuterAccess(localVariableBinding);
            break;
        } 
      } 
    } 
  }
  
  public void manageSyntheticAccessIfNecessary(BlockScope currentScope, FieldBinding fieldBinding, int index, FlowInfo flowInfo) {
    if ((flowInfo.tagBits & 0x1) != 0)
      return; 
    if (fieldBinding.constant((Scope)currentScope) != Constant.NotAConstant)
      return; 
    if (fieldBinding.isPrivate()) {
      FieldBinding codegenField = getCodegenBinding((index < 0) ? ((this.otherBindings == null) ? 0 : this.otherBindings.length) : index);
      ReferenceBinding declaringClass = codegenField.declaringClass;
      if (TypeBinding.notEquals((TypeBinding)declaringClass, (TypeBinding)currentScope.enclosingSourceType())) {
        setSyntheticAccessor(fieldBinding, index, ((SourceTypeBinding)declaringClass).addSyntheticMethod(codegenField, (index >= 0), false));
        currentScope.problemReporter().needToEmulateFieldAccess(codegenField, (ASTNode)this, (index >= 0));
        return;
      } 
    } else if (fieldBinding.isProtected()) {
      int depth = (index == 0 || (index < 0 && this.otherDepths == null)) ? ((
        this.bits & 0x1FE0) >> 5) : 
        this.otherDepths[(index < 0) ? (this.otherDepths.length - 1) : (index - 1)];
      if (depth > 0 && fieldBinding.declaringClass.getPackage() != currentScope.enclosingSourceType().getPackage()) {
        FieldBinding codegenField = getCodegenBinding((index < 0) ? ((this.otherBindings == null) ? 0 : this.otherBindings.length) : index);
        setSyntheticAccessor(fieldBinding, index, (
            (SourceTypeBinding)currentScope.enclosingSourceType().enclosingTypeAt(depth)).addSyntheticMethod(codegenField, (index >= 0), false));
        currentScope.problemReporter().needToEmulateFieldAccess(codegenField, (ASTNode)this, (index >= 0));
        return;
      } 
    } 
  }
  
  public Constant optimizedBooleanConstant() {
    switch (this.resolvedType.id) {
      case 5:
      case 33:
        if (this.constant != Constant.NotAConstant)
          return this.constant; 
        switch (this.bits & 0x7) {
          case 1:
            if (this.otherBindings == null)
              return ((FieldBinding)this.binding).constant(); 
          case 2:
            return this.otherBindings[this.otherBindings.length - 1].constant();
        } 
        break;
    } 
    return Constant.NotAConstant;
  }
  
  public TypeBinding postConversionType(Scope scope) {
    BaseTypeBinding baseTypeBinding;
    TypeBinding typeBinding1, convertedType = this.resolvedType;
    TypeBinding requiredGenericCast = getGenericCast((this.otherBindings == null) ? 0 : this.otherBindings.length);
    if (requiredGenericCast != null)
      convertedType = requiredGenericCast; 
    int runtimeType = (this.implicitConversion & 0xFF) >> 4;
    switch (runtimeType) {
      case 5:
        baseTypeBinding = TypeBinding.BOOLEAN;
        break;
      case 3:
        baseTypeBinding = TypeBinding.BYTE;
        break;
      case 4:
        baseTypeBinding = TypeBinding.SHORT;
        break;
      case 2:
        baseTypeBinding = TypeBinding.CHAR;
        break;
      case 10:
        baseTypeBinding = TypeBinding.INT;
        break;
      case 9:
        baseTypeBinding = TypeBinding.FLOAT;
        break;
      case 7:
        baseTypeBinding = TypeBinding.LONG;
        break;
      case 8:
        baseTypeBinding = TypeBinding.DOUBLE;
        break;
    } 
    if ((this.implicitConversion & 0x200) != 0)
      typeBinding1 = scope.environment().computeBoxingType((TypeBinding)baseTypeBinding); 
    return typeBinding1;
  }
  
  public StringBuffer printExpression(int indent, StringBuffer output) {
    for (int i = 0; i < this.tokens.length; i++) {
      if (i > 0)
        output.append('.'); 
      output.append(this.tokens[i]);
    } 
    return output;
  }
  
  public TypeBinding reportError(BlockScope scope) {
    if (this.binding instanceof ProblemFieldBinding) {
      scope.problemReporter().invalidField(this, (FieldBinding)this.binding);
    } else if (this.binding instanceof ProblemReferenceBinding || this.binding instanceof org.eclipse.jdt.internal.compiler.lookup.MissingTypeBinding) {
      scope.problemReporter().invalidType((ASTNode)this, (TypeBinding)this.binding);
    } else {
      scope.problemReporter().unresolvableReference(this, this.binding);
    } 
    return null;
  }
  
  public TypeBinding resolveType(BlockScope scope) {
    this.actualReceiverType = (TypeBinding)scope.enclosingReceiverType();
    this.constant = Constant.NotAConstant;
    if ((this.binding = scope.getBinding(this.tokens, this.bits & 0x7, (InvocationSite)this, true)).isValidBinding()) {
      TypeBinding type;
      switch (this.bits & 0x7) {
        case 3:
        case 7:
          if (this.binding instanceof LocalVariableBinding) {
            this.bits &= 0xFFFFFFF8;
            this.bits |= 0x2;
            LocalVariableBinding local = (LocalVariableBinding)this.binding;
            if (!local.isFinal() && (this.bits & 0x80000) != 0 && 
              (scope.compilerOptions()).sourceLevel < 3407872L)
              scope.problemReporter().cannotReferToNonFinalOuterLocal((LocalVariableBinding)this.binding, (ASTNode)this); 
            if (local.type != null && (local.type.tagBits & 0x80L) != 0L)
              return null; 
            this.resolvedType = getOtherFieldBindings(scope);
            if (this.resolvedType != null && (this.resolvedType.tagBits & 0x80L) != 0L) {
              FieldBinding lastField = this.otherBindings[this.otherBindings.length - 1];
              scope.problemReporter().invalidField(this, (FieldBinding)new ProblemFieldBinding(lastField.declaringClass, lastField.name, 1), this.tokens.length, this.resolvedType.leafComponentType());
              return null;
            } 
            return this.resolvedType;
          } 
          if (this.binding instanceof FieldBinding) {
            this.bits &= 0xFFFFFFF8;
            this.bits |= 0x1;
            FieldBinding fieldBinding = (FieldBinding)this.binding;
            MethodScope methodScope = scope.methodScope();
            ReferenceBinding declaringClass = (fieldBinding.original()).declaringClass;
            SourceTypeBinding sourceType = methodScope.enclosingSourceType();
            if ((this.indexOfFirstFieldBinding == 1 || (fieldBinding.modifiers & 0x4000) != 0 || (!fieldBinding.isFinal() && declaringClass.isEnum())) && 
              TypeBinding.equalsEquals((TypeBinding)sourceType, (TypeBinding)declaringClass) && 
              methodScope.lastVisibleFieldID >= 0 && 
              fieldBinding.id >= methodScope.lastVisibleFieldID && (
              !fieldBinding.isStatic() || methodScope.isStatic) && (
              !methodScope.insideTypeAnnotation || fieldBinding.id != methodScope.lastVisibleFieldID))
              scope.problemReporter().forwardReference((Reference)this, this.indexOfFirstFieldBinding - 1, fieldBinding); 
            if (isFieldUseDeprecated(fieldBinding, (Scope)scope, (this.indexOfFirstFieldBinding == this.tokens.length) ? this.bits : 0))
              scope.problemReporter().deprecatedField(fieldBinding, (ASTNode)this); 
            if (fieldBinding.isStatic()) {
              if (declaringClass.isEnum() && (
                TypeBinding.equalsEquals((TypeBinding)sourceType, (TypeBinding)declaringClass) || TypeBinding.equalsEquals((TypeBinding)sourceType.superclass, (TypeBinding)declaringClass)) && 
                fieldBinding.constant((Scope)scope) == Constant.NotAConstant && 
                !methodScope.isStatic && 
                methodScope.isInsideInitializerOrConstructor())
                scope.problemReporter().enumStaticFieldUsedDuringInitialization(fieldBinding, (ASTNode)this); 
              if (this.indexOfFirstFieldBinding > 1 && 
                TypeBinding.notEquals((TypeBinding)fieldBinding.declaringClass, this.actualReceiverType) && 
                fieldBinding.declaringClass.canBeSeenBy((Scope)scope))
                scope.problemReporter().indirectAccessToStaticField((ASTNode)this, fieldBinding); 
            } else {
              boolean inStaticContext = (scope.methodScope()).isStatic;
              if (this.indexOfFirstFieldBinding == 1) {
                if (scope.compilerOptions().getSeverity(4194304) != 256)
                  scope.problemReporter().unqualifiedFieldAccess(this, fieldBinding); 
                if (!inStaticContext)
                  scope.tagAsAccessingEnclosingInstanceStateOf(fieldBinding.declaringClass, false); 
              } 
              if (this.indexOfFirstFieldBinding > 1 || 
                inStaticContext) {
                scope.problemReporter().staticFieldAccessToNonStaticVariable((ASTNode)this, fieldBinding);
                return null;
              } 
            } 
            this.resolvedType = getOtherFieldBindings(scope);
            if (this.resolvedType != null && (
              this.resolvedType.tagBits & 0x80L) != 0L) {
              FieldBinding lastField = (this.indexOfFirstFieldBinding == this.tokens.length) ? (FieldBinding)this.binding : this.otherBindings[this.otherBindings.length - 1];
              scope.problemReporter().invalidField(this, (FieldBinding)new ProblemFieldBinding(lastField.declaringClass, lastField.name, 1), this.tokens.length, this.resolvedType.leafComponentType());
              return null;
            } 
            return this.resolvedType;
          } 
          this.bits &= 0xFFFFFFF8;
          this.bits |= 0x4;
        case 4:
          type = (TypeBinding)this.binding;
          type = scope.environment().convertToRawType(type, false);
          return this.resolvedType = type;
      } 
    } 
    return this.resolvedType = reportError(scope);
  }
  
  public void setFieldIndex(int index) {
    this.indexOfFirstFieldBinding = index;
  }
  
  protected void setGenericCast(int index, TypeBinding someGenericCast) {
    if (someGenericCast == null)
      return; 
    if (index == 0) {
      this.genericCast = someGenericCast;
    } else {
      if (this.otherGenericCasts == null)
        this.otherGenericCasts = new TypeBinding[this.otherBindings.length]; 
      this.otherGenericCasts[index - 1] = someGenericCast;
    } 
  }
  
  protected void setSyntheticAccessor(FieldBinding fieldBinding, int index, SyntheticMethodBinding syntheticAccessor) {
    if (index < 0) {
      this.syntheticWriteAccessor = syntheticAccessor;
    } else {
      if (this.syntheticReadAccessors == null)
        this.syntheticReadAccessors = new SyntheticMethodBinding[(this.otherBindings == null) ? 1 : (this.otherBindings.length + 1)]; 
      this.syntheticReadAccessors[index] = syntheticAccessor;
    } 
  }
  
  public void traverse(ASTVisitor visitor, BlockScope scope) {
    visitor.visit(this, scope);
    visitor.endVisit(this, scope);
  }
  
  public void traverse(ASTVisitor visitor, ClassScope scope) {
    visitor.visit(this, scope);
    visitor.endVisit(this, scope);
  }
  
  public String unboundReferenceErrorName() {
    return new String(this.tokens[0]);
  }
  
  public char[][] getName() {
    return this.tokens;
  }
  
  public VariableBinding nullAnnotatedVariableBinding(boolean supportTypeAnnotations) {
    if (this.binding != null && isFieldAccess()) {
      FieldBinding fieldBinding;
      if (this.otherBindings == null) {
        fieldBinding = (FieldBinding)this.binding;
      } else {
        fieldBinding = this.otherBindings[this.otherBindings.length - 1];
      } 
      if (supportTypeAnnotations || fieldBinding.isNullable() || fieldBinding.isNonNull())
        return (VariableBinding)fieldBinding; 
    } 
    return null;
  }
}
